#![allow(dead_code)]
mod ship;
mod system;

pub use ship::{Engine, Gun, Ship};
pub use system::{Object, System};

use anyhow::{bail, Context, Result};
use std::collections::HashMap;
use std::{fs::File, io::Read, path::Path};
use toml;
use walkdir::WalkDir;

mod syntax {
	use super::{ship, system, HashMap};
	use serde::Deserialize;

	#[derive(Debug, Deserialize)]
	pub struct Root {
		pub ship: HashMap<String, ship::syntax::Ship>,
		pub system: HashMap<String, system::syntax::System>,
	}
}

trait Build {
	/// Build a processed System struct from raw serde data
	fn build(root: &syntax::Root) -> Result<Vec<Self>>
	where
		Self: Sized;
}

#[derive(Debug)]
pub struct Content {
	pub systems: Vec<system::System>,
	pub ships: Vec<ship::Ship>,
}

impl Content {
	fn try_parse(path: &Path) -> Result<syntax::Root> {
		let mut file_string = String::new();
		let _ = File::open(path)?.read_to_string(&mut file_string);
		let file_string = file_string.trim();
		return Ok(toml::from_str(&file_string)?);
	}

	fn add_root(&mut self, root: syntax::Root) -> Result<()> {
		let mut p = ship::Ship::build(&root)?;
		for s in &self.ships {
			for o in &p {
				if s.name == o.name {
					bail!(
						"Error parsing content: duplicate ship names {} and {}",
						s.name,
						o.name
					)
				}
			}
		}
		self.ships.append(&mut p);

		let mut p = system::System::build(&root)?;
		for s in &self.systems {
			for o in &p {
				if s.name == o.name {
					bail!(
						"Error parsing content: duplicate system names {} and {}",
						s.name,
						o.name
					)
				}
			}
		}
		self.systems.append(&mut p);

		return Ok(());
	}

	pub fn load_dir(path: &str) -> Result<Self> {
		let mut content = Self {
			systems: Vec::new(),
			ships: Vec::new(),
		};

		for e in WalkDir::new(path).into_iter().filter_map(|e| e.ok()) {
			if e.metadata().unwrap().is_file() {
				// TODO: better warnings
				match e.path().extension() {
					Some(t) => {
						if t.to_str() != Some("toml") {
							println!("[WARNING] {e:#?} is not a toml file, skipping.");
							continue;
						}
					}
					None => {
						println!("[WARNING] {e:#?} is not a toml file, skipping.");
						continue;
					}
				}

				let path = e.path();
				let root = Self::try_parse(path)
					.with_context(|| format!("Could not load {:#?}", e.path()))?;
				content
					.add_root(root)
					.with_context(|| format!("Could not parse {}", path.display()))?;
			}
		}

		return Ok(content);
	}
}
